/*
  Sensor de temperatura remot des d'un servidor web

 creat 18 Dec 2009
 per David A. Mellis
 modificat 4 Sep 2010
 per Tom Igoe
 modificat 7 dec 2011
 per Carles Ferrando
 
 */

#include <SPI.h> //llibreria microcontroladors
#include <Ethernet.h> //libreria comunicacions xarxa
#include <stdio.h> //llibreria estandard
#include <math.h> //libreria de funcions matematiques


byte mac[] = {  0x90, 0xA2, 0xDA, 0x00, 0x64, 0xE3 }; //adreça MAC
byte ip[] = { 192,168,1,15 }; //Adreça IP fixa

Server server(80); //Inicialitzem la llibreria Ethernet amb IP pel port 80

//Definicio de coeficients i constants
float Ventrada=5.0;     // [V]        Tensio d'entrada al divisor de tensio
float Rauxiliar=10000;  // [ohm]      Resistencia secundaria al divisor de tensió.
float R0=10000;    // [ohm]      Valor de NTC nominal a 25ºC
float T0=298.15;   // [K] (25ºC) Valor en ºK de 25ºC
float beta=3891;    // [K] Parametre B
float Rinfinit=0.0214906446;    // [ohm]      Parametre Rinfinit

//Definicio de variables
float Veixida=0.0;    // [V]        Tensio donada pel divisor de tensio
float RNTC=0.0;    // [ohm]      Resistencia del NTC
float TempK=0.0;   // [K]        Temperatura d'eixida en Kelvin
float TempC=0.0;   // [ºC]       Temperatura d'eixida en Celsius
int LecturaNTC = 3; // selecciona el pin d'entrada per al sensor de calor

void setup()
{
  // Iniciem la connexio Ethernet i el servdor
  Ethernet.begin(mac, ip);
  server.begin();
  pinMode(LecturaNTC, INPUT);  //Configuracio del pin d'entrada 
}

//Comença el cicle tancat del programa
void loop()
{
  // escoltem clients d'entrada
  Client client = server.available();
  if (client) {
    // una peticio http acaba amb linia en blanc
    boolean currentLineIsBlank = true;
    while (client.connected()) {
      if (client.available()) {
        char c = client.read();
        // Si heu anat al final de la linia (heu rebut un caracter linia nova)
        // i la linia es en blanc, la peticio ha acabat,
        // ara podeu enviar una resposta
        if (c == '\n' && currentLineIsBlank) {
          // s'esta enviant una capçalera de resposta http
          client.println("HTTP/1.1 200 OK");
          client.println("Content-Type: text/html");
          client.println();

          // eixida amb els valors dels sensors analogics i alguns calculs del programa         
          Veixida=Ventrada*((analogRead(LecturaNTC))/1024.0); //Calcul del valor de voltatge del NTC ara
          RNTC=(Rauxiliar*Veixida/(Ventrada-Veixida)); //Calcul del valor la Resistencia del NTC ara 
          TempK=(beta/log(RNTC/Rinfinit));   //Calcul de la temperatura estimada de SteinHart-Hart en Kelvins
          TempC=TempK-273.15;   //Calcul de la temperatura estimada en Centigrads

          client.print("Temperatura (C)="); //imprimeix el retol en la web del client
          client.print(TempC); //imprimeix la variable TempC en centigrads en la web del client
          break;
        }
        if (c == '\n') {
          // esteu començant una nova linia
          currentLineIsBlank = true;
        } 
        else if (c != '\r') {
          // heu obtes un caracter en la linia actual
          currentLineIsBlank = false;
        }
      }
    }
    // deixem temps al servidor per rebre les dades
    delay(1);
    // tanquem la connexio
    client.stop();
  }
}
